from __future__ import annotations

from typing import Literal, Optional

import pandera as pa

from lixinger import client
from lixinger.config import settings
from lixinger.utils import api, convert_column_list_to_str, get_response_df


class Output(pa.DataFrameModel):
    name: pa.typing.Series[str] = pa.Field(nullable=True)
    stock_code: pa.typing.Series[str]
    area_code: pa.typing.Series[str]
    market: pa.typing.Series[str]
    fs_type: pa.typing.Series[str]
    mutual_markets: Optional[pa.typing.Series[str]] = pa.Field(nullable=True)
    ipo_date: pa.typing.Series[pa.typing.DateTime] = pa.Field(nullable=True)
    delisted_date: Optional[pa.typing.Series[pa.typing.DateTime]] = pa.Field(
        nullable=True
    )


@api
def get_company(
    fs_type: Literal[
        "non_financial", "bank", "insurance", "security", "other_financial"
    ]
    | None = None,
    mutual_markets: Literal["ha"] | None = None,
    stock_codes: list[str] | None = None,
    include_delisted: bool | None = None,
) -> pa.typing.DataFrame[Output]:
    """获取股票详细信息.

    参考文档: https://www.lixinger.com/open/api/doc?api-key=cn/company
    """
    payload = {
        "token": settings.token,
    }
    if fs_type is not None:
        payload["fsType"] = fs_type
    if mutual_markets is not None:
        payload["mutualMarkets"] = mutual_markets
    if stock_codes is not None:
        payload["stockCodes"] = stock_codes
    if include_delisted is not None:
        payload["includeDelisted"] = include_delisted

    response = client.post(
        f"{settings.base_url}/cn/company",
        json=payload,
    )
    df = get_response_df(response, Output)

    if "mutual_markets" in df.columns:
        df["mutual_markets"] = [
            convert_column_list_to_str(column) for column in df["mutual_markets"]
        ]
    return df



from __future__ import annotations

import pandas as pd
import pandera as pa

from lixinger import client
from lixinger.config import settings
from lixinger.utils import api, get_response_df


class Output(pa.DataFrameModel):
    date: pa.typing.Series[pa.typing.DateTime]
    bonus_shares_from_profit: pa.typing.Series[int]
    bonus_shares_from_capital_reserve: pa.typing.Series[int]
    dividend: pa.typing.Series[float]
    content: pa.typing.Series[str] = pa.Field(nullable=True)
    register_date: pa.typing.Series[pa.typing.DateTime] = pa.Field(nullable=True)
    ex_date: pa.typing.Series[pa.typing.DateTime] = pa.Field(nullable=True)
    payment_date: pa.typing.Series[pa.typing.DateTime] = pa.Field(nullable=True)
    status: pa.typing.Series[str]
    original_value: pa.typing.Series[float]
    split_ratio: pa.typing.Series[float] = pa.Field(nullable=True)


@api
def get_dividend_and_alloment(
    start_date: str,
    stock_code: str,
    end_date: str | None = None,
    limit: int | None = None,
) -> pa.typing.DataFrame[Output]:
    """获取分红送配信息.

    参考文档: https://www.lixinger.com/open/api/doc?api-key=cn/company/dividend-and-alloment
    """
    payload = {
        "token": settings.token,
        "startDate": start_date,
        "stockCode": stock_code,
    }
    if end_date is not None:
        payload["endDate"] = end_date
    if limit is not None:
        payload["limit"] = limit

    response = client.post(
        f"{settings.base_url}/cn/company/dividend-and-alloment",
        json=payload,
    )
    df = get_response_df(response, Output)
    df["date"] = pd.to_datetime(df["date"]).dt.tz_localize(None) + pd.Timedelta("8h")
    df["ex_date"] = pd.to_datetime(df["ex_date"]).dt.tz_localize(None) + pd.Timedelta(
        "8h"
    )
    return df



from __future__ import annotations

import pandera as pa

from lixinger import client
from lixinger.config import settings
from lixinger.utils import api, get_response_df


class Output(pa.DataFrameModel):
    area_code: pa.typing.Series[str]
    stock_code: pa.typing.Series[str]
    source: pa.typing.Series[str]


@api
def get_indices(
    stock_code: str,
    date: str | None = None,
) -> pa.typing.DataFrame[Output]:
    """获取股票所属指数信息.

    参考文档: https://www.lixinger.com/open/api/doc?api-key=cn/company/indices
    """
    payload = {
        "token": settings.token,
        "stockCode": stock_code,
    }
    if date is not None:
        payload["date"] = date

    response = client.post(
        f"{settings.base_url}/cn/company/indices",
        json=payload,
    )
    return get_response_df(response, Output)



from __future__ import annotations

import pandera as pa

from lixinger import client
from lixinger.config import settings
from lixinger.utils import api, get_response_df


class Output(pa.DataFrameModel):
    date: pa.typing.Series[pa.typing.DateTime]
    pledgor: pa.typing.Series[str] = pa.Field(nullable=True)
    pledgee: pa.typing.Series[str] = pa.Field(nullable=True)
    pledge_matters: pa.typing.Series[str] = pa.Field(nullable=True)
    pledge_shares_nature: pa.typing.Series[str] = pa.Field(nullable=True)
    pledge_amount: pa.typing.Series[float] = pa.Field(nullable=True)
    pledge_percentage_of_total_equity: pa.typing.Series[float] = pa.Field(nullable=True)
    pledge_start_date: pa.typing.Series[pa.typing.DateTime] = pa.Field(nullable=True)
    pledge_end_date: pa.typing.Series[pa.typing.DateTime] = pa.Field(nullable=True)
    pledge_discharge_date: pa.typing.Series[pa.typing.DateTime] = pa.Field(
        nullable=True
    )
    pledge_discharge_explanation: pa.typing.Series[str] = pa.Field(nullable=True)
    pledge_discharge_amount: pa.typing.Series[float] = pa.Field(nullable=True)
    is_pledge_repurchase_transactions: pa.typing.Series[bool] = pa.Field(nullable=True)
    accumulated_pledge_percentage_of_total_equity: pa.typing.Series[float] = pa.Field(
        nullable=True
    )


@api
def get_pledge(
    start_date: str,
    stock_code: str,
    end_date: str | None = None,
    limit: int | None = None,
) -> pa.typing.DataFrame[Output]:
    """获取股权质押数据.

    参考文档: https://www.lixinger.com/open/api/doc?api-key=cn/company/pledge
    """
    payload = {
        "token": settings.token,
        "startDate": start_date,
        "stockCode": stock_code,
    }
    if end_date is not None:
        payload["endDate"] = end_date
    if limit is not None:
        payload["limit"] = limit

    response = client.post(
        f"{settings.base_url}/cn/company/block-deal",
        json=payload,
    )
    df = get_response_df(response, Output)
    return df



from __future__ import annotations

import pandera as pa

from lixinger import client
from lixinger.config import settings
from lixinger.utils import api, get_response_df


class Output(pa.DataFrameModel):
    name: pa.typing.Series[str] = pa.Field(nullable=True)
    stock_code: pa.typing.Series[str]
    area_code: pa.typing.Series[str]
    market: pa.typing.Series[str]
    source: pa.typing.Series[str] = pa.Field(isin=["csi", "cni", "hsi", "usi", "lxri"])
    currency: pa.typing.Series[str]
    series: pa.typing.Series[str] = pa.Field(
        isin=["size", "sector", "style", "thematic", "strategy"]
    )
    launch_date: pa.typing.Series[pa.typing.DateTime] = pa.Field(nullable=True)
    rebalancing_frequency: pa.typing.Series[str] = pa.Field(
        isin=[
            "annually",
            "semi-annually",
            "quarterly",
            "monthly",
            "irregularly",
            "aperiodically",
        ]
    )
    caculation_method: pa.typing.Series[str] = pa.Field(
        isin=[
            "paasche",
            "grading_weighted",
            "dividend_grading",
            "equal",
            "free_float_cap",
            "modified_cap_weighted",
        ]
    )


@api
def get_index(
    stock_codes: list[str] | None = None,
) -> pa.typing.DataFrame[Output]:
    """获取指数详细信息.

    参考文档: https://www.lixinger.com/open/api/doc?api-key=cn/index
    """
    payload = {"token": settings.token, "stock_codes": stock_codes}

    response = client.post(
        f"{settings.base_url}/cn/index",
        json=payload,
    )
    return get_response_df(response, Output)



from lixinger.api.cn.fund.total_net_value import get_total_net_value


def test_get_index_fundamental():
    get_total_net_value(
        start_date="2023-04-20",
        end_date="2023-04-30",
        stock_code="163406",
    )



from lixinger.api.cn.company.senior_executive_shares_change import (
    get_senior_executive_shares_change,
)


def test_shareholders_num() -> None:
    get_senior_executive_shares_change(start_date="2021-01-01", stock_code="300390")



from __future__ import annotations

import pandera as pa

from lixinger import client
from lixinger.config import settings
from lixinger.utils import api, get_response_df


class Output(pa.DataFrameModel):
    date: pa.typing.Series[pa.typing.DateTime]
    total_net_value: pa.typing.Series[float]


@api
def get_total_net_value(
    start_date: str,
    stock_code: str,
    end_date: str | None = None,
    limit: int | None = None,
) -> pa.typing.DataFrame[Output]:
    """获取基金累计净值数据

    参考文档: https://www.lixinger.com/open/api/doc?api-key=cn/fund/total-net-value
    """
    payload = {
        "token": settings.token,
        "startDate": start_date,
        "stockCode": stock_code,
    }
    if end_date is not None:
        payload["endDate"] = end_date
    if limit is not None:
        payload["limit"] = limit

    response = client.post(
        f"{settings.base_url}/cn/fund/total-net-value",
        json=payload,
    )
    df = get_response_df(response, Output)
    return df



from lixinger.api.cn.company.dividend_and_alloment import (
    get_dividend_and_alloment,
)


def test_get_dividend_and_alloment() -> None:
    get_dividend_and_alloment(start_date="2021-01-01", stock_code="600519")
